package be.dc.aop.testing;

/**
 * Insert class comments here.
 * 
 * @author ruvankeirsbilck
 * @since 8-jul-2003 (16:04:12)
 * @version 1.0
 */

import java.util.*;

public class AOPPerformanceTester {
	public static void main(String[] args) {
		new AOPPerformanceTester ().run (args);
	}
	
	private void run (String[] args) {
		Screen screen = new Screen ();
		Point p1 = new Point (10, 10);
		Point p2 = new Point (20, 20);
		Line line = new Line (p1, p2);
		long elapsedTime1 = this.timeDisplayPoints (screen, p1);
		long elapsedTime2 = this.timeDisplayGraphicalElement (screen, p1);
		long elapsedTime3 = this.timeDisplayGraphicalElementUsingAOP (screen, p1);
		long elapsedTime4 = this.timeDisplayGraphicalElementUsingDoubleDispatching (screen, p1);
		System.out.println ("Testing display of points using immediate method dispatching:" + elapsedTime1);
		System.out.println ("Testing display of points using double dispatching:" + elapsedTime4);
		System.out.println ("Testing display of graphical elements using instanceof and casting:" + elapsedTime2);
		System.out.println ("Testing display of graphical elements using AOP:" + elapsedTime3);
	}

/* Keep out! */
/* Constants */
	private static final int LOOP_COUNT = 1000000;
	private static final int HALF_A_LOOP_COUNT = LOOP_COUNT / 2;

/* Measuring speed */
	private long timeDisplayPoints (Screen screen, Point point) {
		long startTime = System.currentTimeMillis ();
		for (int i = 0; i < LOOP_COUNT; i++) {
			screen.display (point);
		  }
		long stopTime = System.currentTimeMillis ();
		return stopTime - startTime;
	}

	private long timeDisplayGraphicalElement (Screen screen, GraphicalElement graphicalElement) {
		long startTime = System.currentTimeMillis ();
		for (int i = 0; i < LOOP_COUNT; i++) {
			if (graphicalElement instanceof Point) {
				screen.display ((Point) graphicalElement);
			  }
			else {
				screen.display ((Line) graphicalElement);
			  }
		  }
		long stopTime = System.currentTimeMillis ();
		return stopTime - startTime;
	}

	private long timeDisplayGraphicalElementUsingDoubleDispatching (Screen screen, GraphicalElement graphicalElement) {
		long startTime = System.currentTimeMillis ();
		for (int i = 0; i < LOOP_COUNT; i++) {
			screen.display2 (graphicalElement);
		  }
		long stopTime = System.currentTimeMillis ();
		return stopTime - startTime;
	}

	private long timeDisplayGraphicalElementUsingAOP (Screen screen, GraphicalElement graphicalElement) {
		long startTime = System.currentTimeMillis ();
		for (int i = 0; i < HALF_A_LOOP_COUNT; i++) {
			screen.display (graphicalElement);
		  }
		long stopTime = System.currentTimeMillis ();
		return stopTime - startTime;
	}
}